#include "pixelloader.hpp"


SDL_Color PixelLoader::GetPixel ( SDL_Surface* pSurface , int x , int y )
{
    SDL_Color color ;
    Uint32 col = 0 ;

    //determine position
    char* pPosition = ( char* ) pSurface->pixels ;

    //offset by y
    pPosition += ( pSurface->pitch * y ) ;

    //offset by y
    pPosition += ( pSurface->format->BytesPerPixel * x ) ;

    //copy pixel data
    memcpy ( &col , pPosition , pSurface->format->BytesPerPixel ) ;

    //convert color
    SDL_GetRGB ( col , pSurface->format , &color.r , &color.g , &color.b ) ;
    return ( color ) ;
}


PixelLoader::PixelLoader(string imagepath, int etilewidth, int etileheight, int extiles, int eytiles,bool esprites):
        tilewidth(etilewidth),
        tileheight(etileheight),
        xtiles(extiles),
        ytiles(eytiles),
        sprites(esprites)
{
    SDL_Surface* loadedtemp;
    loadedtemp=SDL_LoadBMP( imagepath.c_str()) ;
    tilesheet = SDL_DisplayFormat(loadedtemp);
    SDL_FreeSurface( loadedtemp );
    genLists();
}

static inline float lighten(GLubyte f)
{
    return (f+255)/2;
}


static inline float darken(GLubyte f)
{
    return (f)/2;
}

static inline void drawCube(float x, float y, float z, float depth,GLubyte r, GLubyte g, GLubyte b)
{

    glBegin(GL_QUADS);

    //draw front face
    glColor3ub(r,g,b);
    glVertex3f(x,y,z);
    glVertex3f(x,y+1,z);
    glVertex3f(x+1,y+1,z);
    glVertex3f(x+1,y,z);

    //draw top face
    glColor3ub(lighten(r),lighten(g),lighten(b));
    glVertex3f(x,y+1,z);
    glVertex3f(x,y+1,z+depth);
    glVertex3f(x+1,y+1,z+depth);
    glVertex3f(x+1,y+1,z);

    //draw left face
    glColor3ub(darken(r),darken(g),darken(b));
    glVertex3f(x,y,z);
    glVertex3f(x,y,z+depth);
    glVertex3f(x,y+1,z+depth);
    glVertex3f(x,y+1,z);
    glEnd();
}

int PixelLoader::tiletype(int tile)
{
    if (sprites)
    {
        if (tile<=29)
        {
            return 5;
        }
        else
        {
            return 4;
        }
    }
    else
    {
        switch (tile)
        {

        case 0:
        case 1:
        case 2:
            return 0;//5 pixels deep
        case 3:
        case 4:
        case 5:
            return 1;//pattern top
        case 6:
        case 7:
        case 8:
        case 9:
        case 12:
        case 13:
        case 19:
        case 20:
        case 21:
        case 22:
        case 23:
            return 2;//three pixels deep
        case 10:
        case 11:
        case 30:
        case 31:
        case 32:
        case 33:
        case 34:
        case 35:
            return 3;//in midde
        case 14:
            return 6;//behind back layer
        case 15:
            return 7;//behind front layer
        case 16:
        case 17:
        case 18:
        case 24:
        case 25:
        case 26:
        case 27:
        case 28:
        case 29:
        default:
            return -1;//nothing;
        };
    }

}

void PixelLoader::genLists()
{
    tilelist=glGenLists(xtiles*ytiles);
    for (int i=0;i<xtiles*ytiles;i++)
    {
        glNewList (tilelist+i, GL_COMPILE);
        drawTileIM(0,0,i);
        glEndList();
    }
}

PixelLoader::~PixelLoader()
{
    glDeleteLists( tilelist,xtiles*ytiles );
}

void PixelLoader::drawTile(int x, int y, int tile)
{
    if (tile>=0)
    {
        glTranslatef(x,y,0);
        glCallList(tilelist+tile);
        glTranslatef(-x,-y,0);
    }
}

//MAKE THE Z COORDINATE TILE_DEPENDENT
void PixelLoader::drawTileIM(int x, int y, int tile)
{
    switch (tiletype(tile))
    {
    case 0://SEVEN PIXELS DEEP
        for (int i=0;i<tilewidth;i++)
        {
            for (int j=0;j<tileheight;j++)
            {
                SDL_Color c = GetPixel(tilesheet,(tile%xtiles)*tilewidth+i,(tile/xtiles)*tileheight+j);
                if (c.r!=0 || c.g!=0 || c.b!=0)
                {
                    drawCube(x+i,y-j,1,7,c.r,c.g,c.b);
                }
            }
        }
        break;
    case 1://PATTERNED
        for (int i=0;i<tilewidth;i++)
        {
            for (int j=0;j<tileheight;j++)
            {
                SDL_Color c = GetPixel(tilesheet,(tile%xtiles)*tilewidth+i,(tile/xtiles)*tileheight+j);
                if (c.r!=0 || c.g!=0 || c.b!=0)
                {
                    //DRAW FRONT
                    drawCube(x+i,y-j,1,1,c.r,c.g,c.b);
                    //DRAW BACK
                    drawCube(x+i,y-j,1+(tilewidth-1),1,c.r,c.g,c.b);
                    if (i>0 && i<(tilewidth-1))
                    {
//                        drawCube(x-i,y-j,1,1,c.r,c.g,c.b);
                        //DRAW LEFT
                        drawCube(x,y-j,1+i,1,c.r,c.g,c.b);

                        //DRAW RIGHT
                        drawCube(x+(tilewidth-1),y-j,1+i,1,c.r,c.g,c.b);

                        if (j>0 && j<(tileheight-1))
                        {
                            //DRAW TOP
                            drawCube(x+i,y,1+j,1,c.r,c.g,c.b);

                            //DRAW BOTTOM
                            drawCube(x+i,y-(tilewidth-1),1+j,1,c.r,c.g,c.b);

                        }
                    }

                }
            }
        }
        break;
    case 2:
        for (int i=0;i<tilewidth;i++)
        {
            for (int j=0;j<tileheight;j++)
            {
                SDL_Color c = GetPixel(tilesheet,(tile%xtiles)*tilewidth+i,(tile/xtiles)*tileheight+j);
                if (c.r!=0 || c.g!=0 || c.b!=0)
                {
                    drawCube(x+i,y-j,3,3,c.r,c.g,c.b);
                }
            }
        }
        break;
    case 3:
        for (int i=0;i<tilewidth;i++)
        {
            for (int j=0;j<tileheight;j++)
            {
                SDL_Color c = GetPixel(tilesheet,(tile%xtiles)*tilewidth+i,(tile/xtiles)*tileheight+j);
                if (c.r!=0 || c.g!=0 || c.b!=0)
                {
                    drawCube(x+i,y-j,4,1,c.r,c.g,c.b);
                }
            }
        }
        break;
    case 4:
        for (int i=0;i<tilewidth;i++)
        {
            for (int j=0;j<tileheight;j++)
            {
                SDL_Color c = GetPixel(tilesheet,(tile%xtiles)*tilewidth+i,(tile/xtiles)*tileheight+j);
                if (c.r!=0 || c.g!=0 || c.b!=0)
                {
                    drawCube(x+i,y-j,3,1,c.r,c.g,c.b);
                }
            }
        }
        break;
    case 5:
        for (int i=0;i<tilewidth;i++)
        {
            for (int j=0;j<tileheight;j++)
            {
                SDL_Color c = GetPixel(tilesheet,(tile%xtiles)*tilewidth+i,(tile/xtiles)*tileheight+j);
                if (c.r!=0 || c.g!=0 || c.b!=0)
                {
                    drawCube(x+i,y-j,5,1,c.r,c.g,c.b);
                }
            }
        }
        break;
    case 6:
        for (int i=0;i<tilewidth;i++)
        {
            for (int j=0;j<tileheight;j++)
            {
                SDL_Color c = GetPixel(tilesheet,(tile%xtiles)*tilewidth+i,(tile/xtiles)*tileheight+j);
                if (c.r!=0 || c.g!=0 || c.b!=0)
                {
                    drawCube(x+i,y-j,6,1,c.r,c.g,c.b);
                }
            }
        }
        break;
    case 7:
        for (int i=0;i<tilewidth;i++)
        {
            for (int j=0;j<tileheight;j++)
            {
                SDL_Color c = GetPixel(tilesheet,(tile%xtiles)*tilewidth+i,(tile/xtiles)*tileheight+j);
                if (c.r!=0 || c.g!=0 || c.b!=0)
                {
                    drawCube(x+i,y-j,4,1,c.r,c.g,c.b);
                }
            }
        }
        break;
    default://NO TILE
        break;
    }
}

