#ifndef ENTITIES_HPP
#define ENTITIES_HPP

#include <vector>
using namespace std;

#include "SDL/SDL.h"
#include "SDL/SDL_opengl.h"

#include "Engine/font.hpp"
#include "Engine/audio.hpp"




const int ledgespeed=5;
const int tilesperline=15;
enum Direction {d_up=0,d_down=1,d_left=2,d_right=3};

enum EntityType {e_player,e_heart,e_spike,e_platform};

class Entity
{
public:
    EntityType etype;
    int tile;
    int x;
    int y;
    int width;
    int height;
    virtual void move(){};
    SoundEngine* soundengine;
    Entity(int ex, int ey,int ewidth, int eheight,EntityType eetype,int etile,SoundEngine* esoundengine):
    etype(eetype),tile(etile),x(ex),y(ey),width(ewidth),height(eheight),soundengine(esoundengine){};
};

class Map
{
public:
    bool mask[30][30];
    Entity* player[2];
    vector<Entity*> platforms;
};

//entity with map...
class MEntity:public Entity
{
public:
    Map* map;
    MEntity(int ex, int ey,int ewidth, int eheight,EntityType eetype,int etile,Map* emap,SoundEngine* esoundengine):
            Entity(ex,ey,ewidth,eheight,eetype,etile,esoundengine),map(emap){};

};


enum PlayerState {ps_standing,ps_walking,ps_crouching,ps_jumping,ps_falling,ps_landing,ps_ledgegrab,ps_dying};

class Player:public MEntity
{
public:
    PlayerState playerstate;
    bool onplatform;
    bool num;
    bool facingl;
    bool* dirs;
    void move();
    void kill();
    int statecounter;
    Player(int ex, int ey, bool eplayer,bool* edirs,Map* emap,SoundEngine* esoundengine):
            MEntity(ex,ey,4,6,e_player,(eplayer?0:2*tilesperline)+2,emap,esoundengine),
            playerstate(ps_standing),
            onplatform(false),
            num(eplayer),
            facingl(false),
            dirs(edirs),
            statecounter(0),
            startx(ex),
            starty(ey),
            frame(0)
    {};
private:
    void respawn();
    int startx;
    int starty;
    int frame;
};

class Platform:public MEntity
{
public:
    bool evertical;
    Direction d;
    void move();
    int ticker;
    bool carrying[2];
    void carry(int i);
    bool turning;
    Platform(int ex, int ey,Direction ed, Map* emap,SoundEngine* esoundengine):
            MEntity(ex,ey,7,3,e_platform,8,emap,esoundengine),
            d(ed),ticker(0),turning(false){
            carrying[0]=false;
            carrying[1]=false;};
};

class Spikes:public MEntity
{
    public:
//SPIKES ARE INVISIBLE
    Spikes(int ex, int ey,Map* emap, int tiledir,SoundEngine* esoundengine);
};

class Heart:public MEntity
{
    public:
    bool collected;
    Heart(int ex, int ey,Map* emap, SoundEngine* esoundengine);
};

#endif /* ENTITIES_HPP */
